from gtk import *
import types
import string
import file

debug=0

tooltips= GtkTooltips()
tooltips.set_delay(500)

entry_vsize=20

def create_tool_tip(dict_entry, entry):
    if dict_entry.has_key('description'):
	tooltips.set_tip(entry, dict_entry['description'], '')
	tooltips.enable()

def create_label(vbox, dict_entry):
    text=dict_entry['name'] + " : "
    if dict_entry.has_key('presence') and dict_entry['presence']=='optional':
	label=GtkCheckButton(text)
    else:
	label = GtkLabel(text)
    al=GtkAlignment(1, 0.5, 0, 0)
    al.set_usize(-1,entry_vsize)
    al.add(label)
    vbox.pack_start(al, expand=FALSE)
    label.show()
    al.show()
    return label

def set_widget_parameters(widget, dict_entry):
    if dict_entry.has_key('size'):
	widget.set_usize(dict_entry['size'],-1)    


def set_entry_parameters(entry, dict_entry):
    set_widget_parameters(entry, dict_entry)
    if dict_entry.has_key('value'):
	entry.set_text(dict_entry['value'])
    if dict_entry.has_key('max_length'):
	entry.set_max_length(dict_entry['max_length'])    

def create_entry(vbox, entry, dict_entry):
    al=GtkAlignment(0, 0.5, 0, 0)
    al.set_usize(-1,entry_vsize)
    al.add(entry)
    vbox.pack_start(al, expand=FALSE)
    create_tool_tip(dict_entry, entry)
    entry.show()
    al.show()    

def create_entry_string(vbox, dict_entry):
    entry = GtkEntry()
    dict_entry['_widget']=entry
    set_entry_parameters(entry, dict_entry)
    create_entry(vbox, entry, dict_entry)
    return entry

def create_entry_filename(vbox, dict_entry):
    entry = GtkEntry()
    dict_entry['_widget']=entry
    set_entry_parameters(entry, dict_entry)
    create_tool_tip(dict_entry, entry)    
    entry.show()

    button=GtkButton("Browse...")

    def browse(button, entry=entry, button=button):
	button.set_sensitive(FALSE)
	file_box=GtkFileSelection("file selection")

	def file_box_cancel(widget, button=button):
	    button.set_sensitive(TRUE)
	
	def file_box_ok(widget, entry=entry, file_box=file_box):
	    entry.set_text(file_box.get_filename())
	    file_box.destroy()
	
	file_box.set_filename(entry.get_text())
	file_box.connect("destroy", file_box_cancel/etc/shells)
	file_box.ok_button.connect("clicked", file_box_ok)
	file_box.cancel_button.connect("clicked", file_box.destroy)
	file_box.show()
    
    button.connect("clicked", browse)
    button.show()
    
    hbox = GtkHBox(spacing=2)
    hbox.pack_start(entry)
    hbox.pack_start(button)
    create_entry(vbox, hbox, dict_entry)
    return hbox

def create_entry_integer(vbox, dict_entry):
    min=int(dict_entry['integer_limits'][0])
    max=int(dict_entry['integer_limits'][1])
    if not dict_entry.has_key('value'):
        dict_entry['value']=`min`
    value=string.atoi(dict_entry['value'])
    spin=GtkSpinButton(GtkAdjustment(value,min,max,1,((max - min)/10)+1), 0.5, 0)
    dict_entry['_widget']=spin
    set_entry_parameters(spin, dict_entry)
    create_entry(vbox, spin, dict_entry)
    return spin

def create_entry_list(vbox, dict_entry):
    combo = GtkCombo()
    list2=[]
    for i in dict_entry['list_values']:
	if type(i)==types.TupleType:
	    list2.append(i[0])
	else:
	    list2.append(i)
    combo.set_popdown_strings(list2)
    dict_entry['_widget']=combo.entry
    set_entry_parameters(combo.entry, dict_entry)
    create_tool_tip(dict_entry, combo.entry)
    create_entry(vbox, combo, dict_entry)
    return combo

def add_entry(left_box, right_box, dict_entry):
    "Add the entry in the container."

    label=create_label(left_box, dict_entry)

    if not dict_entry.has_key('value_type'):
	dict_entry['value_type']='string'
    value_type=dict_entry['value_type']
    entry_type=type(value_type)
    if value_type=='string':
	entry=create_entry_string(right_box, dict_entry)
    elif value_type=='integer':
	entry=create_entry_integer(right_box, dict_entry)
    elif value_type=='list':
	entry=create_entry_list(right_box, dict_entry)
    elif value_type=='filename':
	entry=create_entry_filename(right_box, dict_entry)
    else:
	print "Error, unknown value type '"+value_type+"' for entry '"+\
	      dict_entry['name']+"'."
	raise SystemExit, 8

    if not dict_entry.has_key('value'):
	dict_entry['_active']=0
    
    if (dict_entry.has_key('presence') and dict_entry['presence']=='optional'):
	def check_button_clicked(button, entry=entry, dict_entry=dict_entry):
	    active=button.get_active()
	    entry.set_sensitive(active)
	    dict_entry['_active']=active

	label.connect("clicked", check_button_clicked)
	label.set_active(dict_entry['_active'])
	check_button_clicked(label)

def get_widget_value(entry):
    widget_type=entry['_widget'].get_name()
    value=entry['_widget'].get_text()
    if entry['value_type']=='list':
	for i in entry['list_values']:
	    if type(i)==types.TupleType:
		if i[0]==value:
		    value=i[1]
		    break
    if type(value)!=types.StringType:
	value=`value`
    return value

def get_entries_values(widget, parameters):
    global_parameter=parameters[0]
    entries=parameters[1]
    for entry in entries.keys():
        if debug:
            print entries[entry]['name']
            print get_widget_value(entries[entry])
	entries[entry]['value']=get_widget_value(entries[entry])
    file.write_file(parameters)
    mainquit()

def config_window(config, entries):
    window = new(GtkWindow, type=WINDOW_TOPLEVEL, title="LUCI",
		 allow_grow=TRUE, allow_shrink=FALSE, border_width=10)
    window.connect("destroy", mainquit)    
    
    main_vbox = GtkVBox(spacing=2)
    window.add(main_vbox)   

    hbox = GtkHBox(spacing=2)
    left_vbox = GtkVBox(spacing=5)
    right_vbox = GtkVBox(spacing=5)
    hbox.pack_start(left_vbox)
    hbox.pack_start(right_vbox)
    
    if config.has_key('entries'):
	list_entries=config['entries']
    else:
	list_entries=entries.keys()
    for entry in list_entries:
	add_entry(left_vbox, right_vbox, entries[entry])

    buttons_hbox = GtkHBox(spacing=6)
    buttons_hbox.set_border_width(10)
    
    button_OK=GtkButton('OK')
    button_OK.connect("clicked", get_entries_values, (config, entries))
    buttons_hbox.pack_start(button_OK)

    button_cancel=GtkButton('Cancel')
    button_cancel.connect("clicked", mainquit)
    buttons_hbox.pack_start(button_cancel)

    button_OK.show()
    button_cancel.show()

    
    if (config.has_key('title')):
        if config['title']:
            frame=GtkFrame(config['title'])
        else:
            frame=GtkFrame()
        frame.set_border_width(5)
        hbox.set_border_width(10)
        frame.add(hbox)
        main_vbox.pack_start(frame)
        frame.show()
    else:
        main_vbox.pack_start(hbox)
        
    main_vbox.pack_start(buttons_hbox)

    left_vbox.show()
    right_vbox.show()
    hbox.show()
    buttons_hbox.show()
    main_vbox.show()
    window.show()
    mainloop()
